/*
    Tool to create, restore and manage image creation for lab machines

*/

#include <string>

using namespace std ;
#include <iostream>
#include <fstream>
#include <vector>

class OS {
public:
    string OperatingSystem;
    string Location;
    string Remarks;

};

vector <OS> ReadFile() {
    vector <OS> v1;
    string s1,s2;
    int place;

    ifstream myFile;
    myFile.open( "Configuration" );
    if ( myFile.fail() ) {
        cerr << "Configuration file doesn't exist\n";
        exit(1);
    }
    while(1) {
        getline(myFile,s1, '\n');
        if (myFile.rdstate() & ios::failbit) {
            break;
        }
        //cout << s1;
        // Increase the array, and add another OS
        OS o1;
        place = s1.find_first_of(",");
        //cout << "place = " << place;       
        o1.OperatingSystem = s1.substr(0 ,place) ;
        s1 = s1.substr(place+1,1000);
        //cout << "o1.OperatingSystem = " << o1.OperatingSystem << "\n";
        //  cout << s1;
        place = s1.find_first_of(",");
        //cout << "place = " << place << "\n";       
        o1.Location = s1.substr(0 ,place) ;
        s1 = s1.substr(place+1,1000);
        //cout << "o1.Location = " << o1.Location << "\n";
        o1.Remarks = s1;

        v1.push_back(o1);


    }


    return v1;
}

void PrintOSs(vector <OS> v1)
{
    int i;
    cout << "Number of operating systems is " << v1.size() << "\n";
    for (i = 0; i < v1.size(); i++) {
        cout << "(" << i << ") OperatingSystem = " << v1[i].OperatingSystem << 
            " Location = " << v1[i].Location << 
            " Remarks = " << v1[i].Remarks << "\n";

    }
}

void ExecuteCommand(string command)
{
    cout << "Executing: " << command << "\n";

#ifndef WIN32
 system (command.c_str());
#endif
}

string ImageName(vector <OS> v1, int ImageNumber)
{
    if (ImageNumber >= v1.size()) {
        cerr << "No such image " << ImageNumber << "\n";
    }
    string s1 = "Image";
    s1 += v1[ImageNumber].OperatingSystem + "_"+ v1[ImageNumber].Location + "_"+ v1[ImageNumber].Remarks + ".gzip";
    // I want to replace each / with _ to allow creating legal names
    replace_if(s1.begin(), s1.end(), bind2nd(equal_to<char>(), '/'), '_');
    return s1;
}

void CreateImage(vector <OS> v1, int ImageNumber)
{
    string Image = ImageName(v1, ImageNumber);
    string command = "ntfsclone --save-image -o - /dev/";
    command += v1[ImageNumber].Location;
    command += " | gzip -c > ";
    command += Image;
    ExecuteCommand(command);
}

void RestoreImage(vector <OS> v1, int ImageNumber)
{
    string Image = ImageName(v1, ImageNumber);
    string command = "gunzip -c ";
    command += Image;
    command += " | ntfsclone --restore-image --overwrite /dev/";
    command += v1[ImageNumber].Location;
    command += " - ";    
    ExecuteCommand(command);

}

void RebootImage(vector <OS> v1, int ImageNumber)
{
    ExecuteCommand("mount /dev/sda1 /win");
    ExecuteCommand("mount /dev/hda1 /win");
    ExecuteCommand("mount /dev/hdc1 /win");
    string command1 = "touch /win/bootfiles/boot.ini." + v1[ImageNumber].OperatingSystem;    
    ExecuteCommand(command1);
    
    string command = "cp /win/bootfiles/boot.ini." + v1[ImageNumber].OperatingSystem + " /win/boot.ini";    
    ExecuteCommand(command);
}


int  main(int argc, char **argv) 
{
    int Operation, OsNumber;
    bool IsRoot = false;
    vector <OS> v1 = ReadFile();
    PrintOSs(v1);
    cout << "Press 1 to create an image (folowed by image number)\n";
    cout << "Press 2 to restore an image (folowed by image number)\n";
    cout << "Press 3 to restore an image and reboot (folowed by image number)\n";
    cout << "Press 4 to reboot to an image (folowed by image number)\n";
    cout << "Press 5 to create all images (folowed by 0)\n";

    if ((argc == 2) && (!strcmp(argv[1],"/root"))) {
        IsRoot = true;
    }

    if ((argc == 3) && (!strcmp(argv[1],"/RestoreImage"))) {
        OsNumber = atoi(argv[2]);
        cout << "Restoring image " << OsNumber << "\n";
        RestoreImage(v1, OsNumber);
        return 0;
    }


    cin >> Operation >> OsNumber;
    cout << "Operation is " << Operation << " Os number is " << OsNumber << "\n";

    switch (Operation) {
        case 1:
            if (!IsRoot) {
                cout << "must be in root mode to create image,\n use " << argv[0]<< " /root to be in root mode \n";
                exit(3);                
            }
            CreateImage(v1, OsNumber); 
            break;

        case 2: 
            RestoreImage(v1, OsNumber); 
            break;

        case 3:
            RestoreImage(v1, OsNumber);            
            RebootImage(v1, OsNumber); 
            break;

        case 4: 
            RebootImage(v1, OsNumber);
            break;

        case 5:
            {
                if (!IsRoot) {
                    cout << "must be in root mode to create image,\nuse " << argv[0]<< " /root to be in root mode \n";
                    exit(3);                
                }                
                for (int i=0; i < v1.size(); i++) {
                    CreateImage(v1, i); 
                }

            }
            break;
        default:
            cout << "Error Unknown option \n";
            exit(2);
    }


    return 0;
}

